from lcd import *
import machine,pyb,time
from pyb import LED,SPI,UART,I2C
# from pyb import Accel
from pyb import RTC,Pin
from gps import *
import winbond, utime
from bmp180 import BMP180
from network import WIZNET5K
from myfunc import *
from random import randint
import max30102, micropython
    
micropython.alloc_emergency_exception_buf(100)

#pyb.freq(168000000)
pyb.freq(120000000) 
#pyb.freq(96000000)

LED(1).off()
LED(2).off()

tim = pyb.Timer(4)             
tim.init(freq=2) # trigger at 2Hz
tim.callback(lambda t:pyb.LED(2).toggle())

#gc.collect()
#print(gc.mem_free())

# led_strobe()

p_in1 = Pin('X22', Pin.IN,Pin.PULL_NONE)
p_in2 = Pin('X21', Pin.IN,Pin.PULL_NONE)
p_in3 = Pin('X20', Pin.IN,Pin.PULL_NONE)
p_in4 = Pin('X19', Pin.IN,Pin.PULL_NONE)
p_in5 = Pin('X18', Pin.IN,Pin.PULL_NONE)

# p_in3.irq(trigger=Pin.IRQ_RISING | Pin.IRQ_FALLING, handler=lambda p:None) # Проснуться
    
# adcall = pyb.ADCAll(12, 0x70000) # 12 bit resolution, internal channels
# adc = pyb.ADCAll(resolution)        # create an ADCAll object
# adc = pyb.ADCAll(resolution, mask)  # create an ADCAll object for selected analog channels
# val = adcall.read_channel(channel)     # read the given channel
# val = adcall.read_core_temp()          # read MCU temperature
# val = adcall.read_core_vbat()          # read MCU VBAT
# val = adcall.read_core_vref()          # read MCU VREF
# val = adcall.read_vref()               # read MCU supply voltage

power_off_on(True) # Включили все модули = all_is_off()

#import network
nic = WIZNET5K(SPI(1),Pin.board.X12,Pin.board.X11)
ip = ('10.128.0.206','255.255.255.0','10.128.0.1','8.8.8.8')
nic.ifconfig(ip)
    
# ------------------- TFT Display Init ----------------------------
    
l = LCD( rate=42000000 )  # rate=42000000                                                  
l.fillMonocolor( BLACK ) 

# game(l,p_in1)

s = l.initCh(color = GREEN, bgcolor=BLACK, font='Arial_14')
s.printLn('Checking FLASH ...',1,1,scale=1)
# s.font = 'Pitch_m23'
time.sleep(1)

# -------------------- Attach FLASH W25Q32 ------------------------

flash = winbond.W25QFlash(machine.SPI(2), machine.Pin('Y5'))
time.sleep(0.1)

#flash.format()         # !!! only required 
#os.VfsFat.mkfs(flash)  # !!! only required 
#flash.copy('/sd/images/MP_powered.bmp','/win/')

os.mount(flash, '/win') 
time.sleep(0.1)

files = os.listdir('/win')

if 'data_temp.txt' not in files: # Если нет такого файла то создать его
    make_json_data_file('temp','data_temp.txt')
if 'data_bar.txt' not in files: # Если нет такого файла то создать его
    make_json_data_file('bar','data_bar.txt')
if 'data_hum.txt' not in files: # Если нет такого файла то создать его
    make_json_data_file('hum','data_hum.txt')

del files
    
l.renderBmp( '/win/micropython.bmp',pos=(0,65) ) # From FLASH W25Q32
l.renderBmp( '/win/MP_powered.bmp',pos=(0,0))    # From FLASH W25Q32
time.sleep(2)

# led_strobe()

# ------------------- I2C ------------------------------------------

l.fillMonocolor( BLACK ) 

bus = I2C(2, I2C.MASTER, baudrate=20000)
i2c_devices = bus.scan()
if 119 in i2c_devices:
    s.printLn('BMP180 is ok.', 1,1,scale=1)
if 87 in i2c_devices:
    s.printLn('MAX30102 is ok.',1,15,scale=1)
        
bmp180 = BMP180(bus)
bmp180.oversample_sett = 3
bmp180.baseline = 101325.0
temp = bmp180.temperature
p = bmp180.pressure
alt = bmp180.altitude
s.printLn('Temp: '+str(int(temp)),    1,30,scale=1)
s.printLn('Press: '+str(int(p/133)+5),  1,45,scale=1)
s.printLn('Alt: '+str(int(alt)),      1,60,scale=1)

# --------------------- RTC --------------------------------------

rtc = RTC()
s.printLn(rtc_show(rtc.datetime()), 1,75,scale=1)

#rtc.datetime((2019, 12, 11, 1, 17, 8, 0, 0)) # set a specific date and time

# ----------------------- Accel ----------------------------------------

from pyb import Accel
accel = Accel()
s.printLn('X: '+str(accel.x())+' Y:'+str(accel.y())+' Z:'+str(accel.z()), 1,90,scale=1)
# 
#       +X 
#        |
# +Y <---  ---> -Y
#        |
#       -X
# between -32 and 31    
s.printLn('FREQ: '+str(pyb.freq()[0]/1000000)+' Mhz', 1,105,scale=1)
del(Accel)

# ----------------------- UART GPS --------------------------------------
        
uart = UART(4, 9600)                         
uart.init(9600, bits=8, parity=None, stop=1,  timeout=3000) # Timeout ОБЯЗАТЕЛЬНО

# --------------------- Test for GPS reacive DATA NMEA

start = time.ticks_ms()
gps_stat = False
gps = ''

while True:
    delta = time.ticks_diff(time.ticks_ms(), start)
    if delta > 10000:
        break
    data = uart.readline()
    try:
        gps = data.decode('utf-8')
    except:
        pass
    gps = gps.rstrip()
    if gps[0:6] == "$GPRMC": 
        gps_stat = True
        break

if gps_stat:
    s.printLn('GPS is OK.', 1,120,scale=1) 
else:
    s.printLn('GPS Error !!!',1,120,scale=1) 
 
del gps

# --------------- Listing when loading --------------------------------

adcall = pyb.ADCAll(12, 0x70000) # 12 bit resolution, internal channels

val = adcall.read_core_temp()          # read MCU temperature
s.printLn('CPU Temp: '+str(val),1,135,scale=1)

val = adcall.read_core_vbat()          # read MCU VBAT
s.printLn('VBAT: '+str(val),1,150,scale=1)

val = adcall.read_core_vref()          # read MCU VREF
s.printLn('Core VREF: '+str(val),1,165,scale=1)

val = adcall.read_vref()               # read MCU supply voltage
s.printLn('VREF: '+str(val),1,180,scale=1)

gc.collect()
s.printLn('Memory FREE: '+str(int(gc.mem_free()/1024)),1,195,scale=1)

if nic.isconnected():
    s.printLn('IP: '+nic.ifconfig()[0],1,210,scale=1)
    s.printLn('Gate: '+nic.ifconfig()[2],1,225,scale=1)
    s.printLn('DNS: '+nic.ifconfig()[3],1,240,scale=1)

if nic.isconnected():
    tm_ntp = getntptime()    
    if tm_ntp != 0:
        tm_ntp = tm_ntp + 60*60*3 # GMT +3
        s.printLn('NTP: OK',1,255,scale=1)
        tm = utime.localtime(tm_ntp)        
        rtc.datetime((tm[0], tm[1], tm[2], tm[6] + 1, tm[3], tm[4], tm[5], 0))
        # print(utime.localtime(ntp_time))

time.sleep(5)
del val

# ----------------------------------------------------------------------

# led_strobe()

l.fillMonocolor( WHITE ) 
h = l.initCh(color=RED, font='Pitch_m23') # Для измерения пульса
l.renderBmp( '/win/earth.bmp',pos=(40,70))
h.printLn('Please Wait ...',10,10,scale=1)

# l.drawRect(20,20,195,150,BLUE,infill=BLACK)   
# grid(l) 

# heart()  # Примерно работает

UpdateInterval = utime.time()
UpdateIntervalR = utime.time()

currentUnixTime = utime.time()
pos = int(((currentUnixTime/1800)%192))
temp = bmp180.temperature
p = bmp180.pressure
hum = 27
alt = bmp180.altitude   

cpu = int(adcall.read_core_temp())

tim1 = pyb.Timer(1)             
tim1.init(freq=3) # trigger at 3Hz
tim1.callback(lambda t:pyb.LED(1).toggle())  
save_data_to_file(pos,currentUnixTime,'temp',int(temp))
save_data_to_file(pos,currentUnixTime,'bar',int(p/133)+5)
save_data_to_file(pos,currentUnixTime,'hum',hum)
tim1.deinit()
LED(1).off()

l.fillMonocolor( BLACK ) 

show_bar(l)
show_real_data(l,temp,p,hum,cpu,alt,rtc_show(rtc.datetime()))
show_temp(l)

rcolor = (randint(128, 255), randint(128, 255), randint(128, 255))
x_progress = 20
l.drawVline(x_progress,5,10,rcolor)
nv = 1 # Куда двигать змейку
gc.collect()

h = l.initCh(color=RED, font='Pitch_m23') # Для измерения пульса
 
del s

SleepMode = False
Display = 0
Reset_Display = False

m = max30102.MAX30102(pin='Y4')
m.shutdown()

tim.deinit()
LED(2).off()

gc.collect()

start = pyb.millis()

while True:
    
    currentUnixTime = utime.time()
        
    if Display == 1 and not SleepMode:
        l.fillMonocolor( WHITE ) 
        l.renderBmp( '/win/heart.bmp' ,pos=(40,70))      
        h.printLn('Press F1 to Start' ,5,10,scale=1)
        h.printLn('Press F2 to Exit'  ,5,40,scale=1)
        h.printLn('Press F3 Next Page',5,70,scale=1)
        
        start = pyb.millis()       
        
        while True: 
            if pyb.elapsed_millis(start) > 9000:
                Reset_Display = True
                Display = 0                
                m.shutdown()                
                break
                
            if p_in1.value():
                time.sleep(0.5)
                heart(m,h)
                m.shutdown()
                #l.drawRect(0,220,240,100,WHITE,infill=WHITE) 
                #l.drawRect(0,70,240,30,WHITE,infill=WHITE)                 
                Reset_Display = True
                Display = 0                 
                
            if p_in2.value(): 
                time.sleep(0.5)
                Reset_Display = True
                Display = 0
                m.shutdown()                
                break    
            
            if p_in3.value(): 
                time.sleep(0.5)
                Display = 2
                m.shutdown()                
                break    
                
    if Display == 2 and not SleepMode:  # Game
        game(l,p_in1)
        Display = 0
        Reset_Display = True
                
    if not SleepMode:
        if pyb.elapsed_millis(start) > 100:        
            start = pyb.millis()
            if nv == 1:
                rcolor = (randint(128, 255), randint(128, 255), randint(128, 255))
                l.drawVline(x_progress,5,10,rcolor,width=5)
                x_progress += 5
                if x_progress > 210: 
                    x_progress = 20
                    nv = 2                
            if nv == 2:
                l.drawVline(x_progress,5,10,BLACK,width=5)
                x_progress += 5
                if x_progress > 210: 
                    x_progress = 210
                    nv = 3                
            if nv == 3:
                rcolor = (randint(128, 255), randint(128, 255), randint(128, 255))
                l.drawVline(x_progress,5,10,rcolor,width=5)
                x_progress -= 5
                if x_progress < 20:
                    x_progress = 210
                    nv = 4        
            if nv == 4:
                l.drawVline(x_progress,5,10,BLACK,width=5)            
                x_progress -= 5
                if x_progress < 20: 
                    x_progress = 20
                    nv = 1

    if Reset_Display:
        Reset_Display = False
        l.fillMonocolor( BLACK ) 
        cpu = int(adcall.read_core_temp())
        temp = bmp180.temperature
        p = bmp180.pressure
        hum = 27
        cpu = int(adcall.read_core_temp())
        alt = bmp180.altitude   
        show_bar(l)
        show_real_data(l,temp,p,hum,cpu,alt,rtc_show(rtc.datetime()))
        show_temp(l)
        
    if(currentUnixTime - UpdateInterval > 900 ):
        UpdateInterval = currentUnixTime
        if not SleepMode:
            tim = pyb.Timer(4)             
            tim.init(freq=2) # trigger at 2Hz
            tim.callback(lambda t:pyb.LED(1).toggle())        
        gc.collect()
        pos = int((utime.time()/1800)%192)        
        temp1 = bmp180.temperature
        temp2 = bmp180.temperature
        temp = (temp1 + temp2)/2
        p1 = bmp180.pressure
        p2 = bmp180.pressure
        p = (p1+p2)/2
        hum = 27          
        save_data_to_file(pos,currentUnixTime,'temp',int(temp))
        save_data_to_file(pos,currentUnixTime,'bar',int(p/133)+5)
        save_data_to_file(pos,currentUnixTime,'hum',hum)            
            
        if not SleepMode: 
            show_bar(l)
            show_temp(l)
            tim.deinit()
            LED(1).off()
            
            # Set Date Time From GPS
            start = time.ticks_ms()        
            while True:
                delta = time.ticks_diff(time.ticks_ms(), start)
                if delta > 10000:
                    break
                data = uart.readline()
                try:
                    gps = data.decode('utf-8')
                except:
                    pass
                gps = gps.rstrip()
                if gps[0:6] == "$GPRMC": 
                    set_date_time(rtc,gps)
                    del gps
                    break

        gc.collect()

    if(currentUnixTime - UpdateIntervalR > 60):      
        UpdateIntervalR = currentUnixTime   
        if not SleepMode:
            gc.collect()
            cpu = int(adcall.read_core_temp())
            temp = bmp180.temperature
            p = bmp180.pressure
            hum = 27
            cpu = int(adcall.read_core_temp())
            alt = bmp180.altitude    
            show_real_data(l,temp,p,hum,cpu,alt,rtc_show(rtc.datetime()))
            gc.collect()

    if p_in1.value(): 
        if not SleepMode:
            break    
    
    if p_in3.value():
        if not SleepMode:
            time.sleep(0.5)
            Display = Display + 1
            if Display > 1: Display = 1
        
    if p_in4.value():            
        time.sleep(0.5)        
        
    if p_in5.value(): 
        time.sleep(0.5)        
        if SleepMode:
            power_off_on(True)
            SleepMode = False
            Reset_Display = True
            nic = WIZNET5K(SPI(1),Pin.board.X12,Pin.board.X11)
            nic.ifconfig(ip)    
            time.sleep(0.5)
            #machine.reset()
        else:
            power_off_on(False)
            SleepMode = True   
            del(nic)
            gc.collect()
            #pyb.freq(96000000)
            time.sleep(0.5)     
    pyb.wfi()
            
'''
while True:
    led_strobe()
    if accel.x() > 0: # Наклонить от себя
        break
    if p_in5.value():
        break
    
'''
    
'''
while True:
    accel = Accel()
    if accel.x() > 15:
        all_is_on()
    if accel.y() > 15:
        break
        
    print(accel.x(), accel.y(), accel.z(), accel.tilt(), accel.filtered_xyz())

'''


'''
while True:
    if p_in1.value():
        print(1)
    if p_in2.value():
        print(2)
    if p_in3.value():
        print(3)
    if p_in4.value():
        print(4)
    if p_in5.value():
        print(5)
       
'''